package eu.asvachin.hashcalculator;

import eu.asvachin.hashcalculator.HashController.HashQueue;
import eu.asvachin.hashcalculator.hashes.Hash;
import java.util.Arrays;

/**
 * Simple runnable wrapper around a hash function so it can be used asynchronously.
 *
 * @author Miles Asvachin
 */
public final class Hasher implements Runnable
{
	private final Hash hash;
	private final HashQueue queue;
	private byte[] data = null;
	private volatile boolean finished = false;
	
	public Hasher(final Hash newHash, final HashQueue newQueue)
	{
		hash = newHash;
		hash.reset();
		queue = newQueue;
	}
	
	public synchronized void update(final byte[] newData, final int length)
	{
		data = Arrays.copyOf(newData, length);
		notifyAll();
	}
	
	public synchronized void finish()
	{
		finished = true;
		hash.finish();
		notifyAll();
	}
	
	@Override
	public void run()
	{
		synchronized(this)
		{
			while(!finished)
			{
				while(data == null && !finished)
				{
					try
					{
						wait();
					}
					catch(InterruptedException ex){}
				}

				if(!finished)
				{
					hash.update(data);
					data = null;
					queue.addWaitingHasher(this);
				}
			}
		}
	}
}
