package eu.asvachin.hashcalculator.gui;

import eu.asvachin.hashcalculator.gui.HashTableModel.ColumnDescription;
import java.awt.Component;
import java.awt.Rectangle;
import java.math.BigInteger;
import java.util.Collection;
import javax.swing.BorderFactory;
import javax.swing.JLabel;
import javax.swing.JTable;
import javax.swing.table.TableCellRenderer;
import javax.swing.table.TableColumn;

/**
 * Sets the widths of the table columns to appropriate values.
 *
 * @author Miles Asvachin
 */
class HashTableHeaderRenderer extends JLabel implements TableCellRenderer
{
	final Collection<JTable> tables;
	
	public HashTableHeaderRenderer(final Collection<JTable> newTables)
	{
		tables = newTables;
		setBorder(BorderFactory.createRaisedBevelBorder());
	}
	
	@Override
	public Component getTableCellRendererComponent(final JTable table, final Object value, final boolean isSelected, final boolean hasFocus, final int rowIndex, final int columnIndex)
	{
        setText(value.toString());

		if(columnIndex == ColumnDescription.HASH_NAME.ordinal() || columnIndex == ColumnDescription.MATCH.ordinal())
		{
			//Not sure why, but it's not quite wide enough to fit the longest text without the +1.
			final int preferredWidth = Math.max(getMaxCellWidth(columnIndex), getPreferredSize().width) + 1;
			final TableColumn column = table.getColumnModel().getColumn(columnIndex);
			column.setPreferredWidth(preferredWidth);
			column.setMaxWidth(preferredWidth);
		}

        return this;
	}
	
	private int getMaxCellWidth(final int columnIndex)
	{
		int maxCellWidth = 0;
		
		for(final JTable table : tables)
		{
			for(int rowIndex = 0; rowIndex < table.getRowCount(); rowIndex++)
			{
				final TableCellRenderer cellRenderer = table.getCellRenderer(rowIndex, columnIndex);
				final Component cellRendererComponent = cellRenderer.getTableCellRendererComponent(
						table,
						table.getModel().getValueAt(rowIndex, columnIndex),
						false,
						false,
						rowIndex,
						columnIndex);
				final int cellWidth = cellRendererComponent.getPreferredSize().width;
				maxCellWidth = Math.max(maxCellWidth, cellWidth);
			}
		}
		
		return maxCellWidth;
	}

	//Overridden for performance reasons.
	@Override
	public void invalidate(){}
	@Override
    public void validate() {}
	@Override
    public void revalidate() {}
	@Override
	public void repaint(long tm, int x, int y, int width, int height) {}
	@Override
    public void repaint(Rectangle r){}
	@Override
    public void repaint(){}
	@Override
    protected void firePropertyChange(String propertyName, Object oldValue, Object newValue) {}
	@Override
    public void firePropertyChange(String propertyName, boolean oldValue, boolean newValue) {}
}
