package eu.asvachin.hashcalculator.hashes;

import java.math.BigInteger;
import org.bouncycastle.crypto.Digest;

/**
 * Wrapper for org.bouncycastle.crypto.Digest
 *
 * @version 1.0 04 October 2006
 * @author Miles Asvachin
 */
public abstract class BCHash extends Hash
{
	protected Digest md;
	
	@Override
	public void reset(){md.reset();}

	@Override
	public void update(final byte[] newData){md.update(newData, 0, newData.length);}

	@Override
	public String getHashName(){return md.getAlgorithmName();}

	@Override
	public HashType getHashType(){return HashType.CRYPTOGRAPHIC;}

	@Override
	public void finish()
	{
		final int digestByteLength = md.getDigestSize();
		final byte[] digestBytes = new byte[digestByteLength + 1];
		final String digestFormat = "%" + "0" + (digestByteLength * 2) + "x";
		
		md.doFinal(digestBytes, 1);
		
		digest = String.format(digestFormat, new BigInteger(digestBytes));
	}
}
