package eu.asvachin.hashcalculator.hashes;

/**
 * A common interface for all hash functions.
 *
 * @author Miles Asvachin
 */
public abstract class Hash
{
	public enum HashType
	{
		CHECKSUM("Checksum"),
		CRC("Cyclic Redundancy Check"),
		CRYPTOGRAPHIC("Cryptographic Hash");

		private final String friendlyName;

		HashType(final String newName)
		{
			friendlyName = newName;
		}
		
		@Override
		public String toString()
		{
			return friendlyName;
		}
	}
	
	/** Name of the hash function implemented by this class. */
	protected String hashName;
	/** Last calculated digest. */
	protected String digest;
	
	/**
	 * Process some data.
	 */
	public abstract void update(final byte[] newData);
	/**
	 * Does whatever is needed to calculate the hash after all data has been processed.
	 * The hash should be in a reset state when this returns.
	 */
	public abstract void finish();
	/**
	 * Discard all current state, ready to calculate another hash.
	 */
	public abstract void reset();
	
	public abstract HashType getHashType();
	public String getHashName(){return hashName;}
	public String getDigest(){return digest == null ? null : digest.toUpperCase();}
	
	@Override
	public String toString(){return getHashName();}
}
