package eu.asvachin.hashcalculator.hashes.crc;

import eu.asvachin.hashcalculator.utilities.BitUtilities;

/**
 * General purpose CRC algorithm with lookup table.
 * Width must be at least 8 and no more than 32 bits.
 *
 * @author Miles Asvachin
 */
public class IntCRC extends SlowCRC
{
	private final int[] lookupTable = new int[256];
	
	protected IntCRC(String name, int w, int poly, int init, boolean refIn, boolean refOut, int xorVal)
	{
		super(name, w, refIn ? BitUtilities.reflect(poly, w) : poly, init, refIn, refIn ^ refOut, xorVal);
		calcTable();
	}
	
	/**
	 * Precalculate values to XOR with register for every possible input byte.
	 */
	private void calcTable()
	{
		int i, j;
		
		if(reflectIn)
		{
			for(i = 0; i < lookupTable.length; i++)
			{
				lookupTable[i] = i ;
				
				for(j = 0; j < 8; j++)
				{
					if((lookupTable[i] & 0x1) != 0)
					{
						lookupTable[i] = (lookupTable[i] >>> 1) ^ polynomial;
					}
					else
					{
						lookupTable[i] >>>= 1;
					}
				}
			}
		}
		else
		{
			for(i = 0; i < lookupTable.length; i++)
			{
				lookupTable[i] = i << (width - 8);
				
				for(j = 0; j < 8; j++)
				{
					if((lookupTable[i] & msb) != 0)
					{
						lookupTable[i] = (lookupTable[i] << 1) ^ polynomial;
					}
					else
					{
						lookupTable[i] <<= 1;
					}
				}
			}
		}
	}
	
	@Override
	public void update(byte[] newData)
	{
		for(final byte b : newData)
		{
			//n.b, the reflected algorithm reflects both input *and* output, so may need to reflect output again.
			if(reflectIn)
			{
				shiftRegister = lookupTable[(shiftRegister ^ b) & 0xFF] ^ (shiftRegister >>> 8);
			}
			else
			{
				shiftRegister = lookupTable[((shiftRegister >> (width - 8)) ^ b) & 0xFF] ^ (shiftRegister << 8);
			}
		}
	}
}