package eu.asvachin.hashcalculator.hashes.sum;

import eu.asvachin.hashcalculator.hashes.Hash;

/**
 * "sum" type checksum up to 64 bits in length.
 *
 * @author Miles Asvachin
 */
public class Sum extends Hash
{
	private long sum = 0;
	private final int sumSize;
	
	/**
	 * @param size the type of sum, e.g. sum8, sum16 etc. up to 64. Must be a multiple of 4.
	 */
	public Sum(int size)
	{
		if(size < 4 || size > 64 || (size % 4 != 0))
		{
			throw new IllegalArgumentException("Size must be a multiple of 4 no more than 64");
		}
		
		sumSize = size;
		hashName = "sum" + size;
	}
	
	@Override
	public void reset(){sum = 0;}

	@Override
	public void update(final byte[] newData)
	{
		for(final byte b : newData)
		{
			sum += b & 0xFF;
		}
	}
	
	@Override
	public void finish()
	{
		//Ensure displayed value fills up the correct number of bits.
		String digestFormat = "%" + "0" + sumSize/4 + "x";
		
		sum %= Math.pow(2, sumSize);
		digest = String.format(digestFormat, sum);
		reset();
	}

	@Override
	public HashType getHashType(){return HashType.CHECKSUM;}
}