package eu.asvachin.hashcalculator.hashes.tiger;

import eu.asvachin.hashcalculator.hashes.BCHash;
import org.bouncycastle.crypto.digests.TigerDigest;

/**
 * Tiger/192
 *
 * @author Miles Asvachin
 */
public class Tiger extends BCHash
{
	private final int byteLength;

	/**
	 * Defaults to the maximum 192 bit width.
	 */
	public Tiger(){this(192);}
	
	/**
	 * @param length result length in bits. Must be a multiple of 8 and no more than 192.
	 */
	public Tiger(int length)
	{
		if(length < 8 || length > 192 || (length % 8 != 0))
		{
			throw new IllegalArgumentException("Length must be a multiple of 8 no more than 192");
		}
		
		md = new TigerDigest();
		byteLength = length / 8;
	}
	
	@Override
	public String getHashName(){return super.getHashName() + "/" + (byteLength * 8);}
	
	@Override
	public void finish()
	{
		super.finish();
		
		digest = digest.substring(0, byteLength * 2);
	}
}